@@ -1,29 +1,21 @@
 /*
     Copyright 2005-2014 Intel Corporation.  All Rights Reserved.
 
-    This file is part of Threading Building Blocks.
+    This file is part of Threading Building Blocks. Threading Building Blocks is free software;
+    you can redistribute it and/or modify it under the terms of the GNU General Public License
+    version 2  as  published  by  the  Free Software Foundation.  Threading Building Blocks is
+    distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
+    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+    See  the GNU General Public License for more details.   You should have received a copy of
+    the  GNU General Public License along with Threading Building Blocks; if not, write to the
+    Free Software Foundation, Inc.,  51 Franklin St,  Fifth Floor,  Boston,  MA 02110-1301 USA
 
-    Threading Building Blocks is free software; you can redistribute it
-    and/or modify it under the terms of the GNU General Public License
-    version 2 as published by the Free Software Foundation.
-
-    Threading Building Blocks is distributed in the hope that it will be
-    useful, but WITHOUT ANY WARRANTY; without even the implied warranty
-    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with Threading Building Blocks; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
-
-    As a special exception, you may use this file as part of a free software
-    library without restriction.  Specifically, if other files instantiate
-    templates or use macros or inline functions from this file, or you compile
-    this file and link it with other files to produce an executable, this
-    file does not by itself cause the resulting executable to be covered by
-    the GNU General Public License.  This exception does not however
-    invalidate any other reasons why the executable file might be covered by
-    the GNU General Public License.
+    As a special exception,  you may use this file  as part of a free software library without
+    restriction.  Specifically,  if other files instantiate templates  or use macros or inline
+    functions from this file, or you compile this file and link it with other files to produce
+    an executable,  this file does not by itself cause the resulting executable to be covered
+    by the GNU General Public License. This exception does not however invalidate any other
+    reasons why the executable file might be covered by the GNU General Public License.
 */
 
 #include "seismic_video.h"
@@ -45,18 +37,18 @@ void SeismicVideo::on_key(int key) {
     else if(char(key) == 'e') updating = true;
     else if(char(key) == 'd') updating = false;
     else if(key == 27) running = false;
-    title = initIsParallel?titles[1]:titles[0];
+    title = titles[initIsParallel?1:0];
 }
 
 void SeismicVideo::on_process() {
     tbb::task_scheduler_init Init(threadsHigh);
-    do {
+    for( int frames = 0; numberOfFrames_==0 || frames<numberOfFrames_; ++frames ) {
         if( initIsParallel )
             u_.ParallelUpdateUniverse();
         else
             u_.SerialUpdateUniverse();
-        if( numberOfFrames_ > 0 ) --numberOfFrames_;
-    } while(next_frame() && numberOfFrames_);
+        if( !next_frame() ) break;
+    }
 }
 
 #ifdef _WINDOWS
@@ -68,7 +60,7 @@ SeismicVideo * gVideo = NULL;
 SeismicVideo::SeismicVideo(    Universe &u, int number_of_frames, int threads_high, bool init_is_parallel)
     :numberOfFrames_(number_of_frames),initIsParallel(init_is_parallel),u_(u),threadsHigh(threads_high)
 {
-    title = initIsParallel?titles[1]:titles[0];
+    title = titles[initIsParallel?1:0];
 #ifdef _WINDOWS
     gVideo = this;
     LoadStringA(video::win_hInstance, IDC_SEISMICSIMULATION, szWindowClass, MAX_LOADSTRING);
